import os

# provide screen resolution to get the offset to center the HUD calculated
screen_width = 3440 
screen_height = 1440  

# List of .lua files to process (without the .lua extension)
lua_files_to_process = [
    "activity_level", "alpha_skin", "building_purchase", "cdl01", "cellphone", "city_load",
    "clothing_store", "completion", "credits", "crib_cust", "cte_news6", "cte_sniper_rifle",
    "cutscene_test", "cutscene_tv", "demo_derby", "dialog_box", "gang_cust", "garage", 
    "gmb_blackjack", "gmb_poker", "hud", "hud_btnmash", "hud_coop_div", "hud_demo_derby", 
    "hud_msg", "hud_multi_prototype", "hud_overhead", "hud_race_start", "hud_touch_combo", 
    "hud_zombie", "liquor_store", "main_menu", "menu_base", "menu_gambling_poker", 
    "menu_multi_color", "menu_style_dialog", "mission_replay", "mm_wireless", "mp_completion", 
    "mp_hud", "mp_hud_lobby", "mp_leaderboards", "mp_lobby_players", "mp_match_load", 
    "mp_player_info_popup", "mp_scoreboard", "mp_server_browser", "music_store", "pause_map", 
    "pause_menu", "pcr01", "ss_grid", "television", "tutorial", "tutorial_subtitles", 
    "vehicle_cust", "vignette", "weapon_store"
]

# Groups that require special handling (like negative anchor and y = -500)
special_groups = {
    "mayhem_grp": {"scale": True, "anchor": True, "negative_anchor": True},
    "vignettes": {"scale": True, "anchor": True, "negative_anchor": True},
    "health_mini_grp": {"anchor": True, "negative_anchor": False, "y_value": -500},
    "health_large_grp": {"anchor": True, "negative_anchor": False, "y_value": -500},
    "extra_homie": {"anchor": True, "negative_anchor": False, "y_value": -500},
    "mp_snatch_john": {"anchor": True, "negative_anchor": False, "y_value": -500},
}

# Function to calculate the offset for anchor - the value centers our HUD
def calculate_offset(screen_width):
    return (screen_width - 1280) / 2

# Function to handle modifications for special groups like mayhem_grp and vignettes
def handle_special_groups(lines, aspect_ratio, reference_width):
    scale_value = 1 / (1.77777777778 / aspect_ratio)
    anchor_value = ((reference_width - 1280) / 2)

    for group, options in special_groups.items():
        # Prepare the scale line if scaling is required
        if options.get("scale", False):
            scale_line = f'vint_set_property(vint_object_find("{group}", 0, vint_document_find("hud")), "scale", {scale_value}, 1)\n'
            modified = False
            for i, line in enumerate(lines):
                if f'vint_set_property(vint_object_find("{group}"' in line and 'scale' in line:
                    lines[i] = scale_line
                    modified = True
                    print(f"Modified scale for {group} in hud.lua: {scale_line.strip()}")
                    break
            if not modified:
                lines.append(scale_line)
                print(f"Added scale for {group} to hud.lua: {scale_line.strip()}")

        # Prepare the anchor line if anchoring is required
        if options.get("anchor", False):
            anchor_x = -anchor_value if options.get("negative_anchor", False) else anchor_value
            anchor_y = options.get("y_value", 0)  # Use y_value if provided, otherwise default to 0
            anchor_line = f'vint_set_property(vint_object_find("{group}", 0, vint_document_find("hud")), "anchor", {anchor_x}, {anchor_y})\n'
            modified = False
            for i, line in enumerate(lines):
                if f'vint_set_property(vint_object_find("{group}"' in line and 'anchor' in line:
                    lines[i] = anchor_line
                    modified = True
                    print(f"Modified anchor for {group} in hud.lua: {anchor_line.strip()}")
                    break
            if not modified:
                lines.append(anchor_line)
                print(f"Added anchor for {group} to hud.lua: {anchor_line.strip()}")

    return lines

# Function to find and modify or append the required line in specified .lua files
def append_or_modify_line_in_lua_files(directory, float_value, files_to_process):
    for filename in os.listdir(directory):
        file_without_extension = filename.replace('.lua', '')
        if file_without_extension in files_to_process:
            file_path = os.path.join(directory, filename)
            with open(file_path, 'r+') as file:
                lines = file.readlines()

                # Prepare the line to be appended or modified with the filename as the variable
                variable_name = file_without_extension
                line_to_append = f'vint_set_property(vint_object_find("safe_frame", 0, vint_document_find("{variable_name}")), "anchor", {float_value}, 0)\n'

                # Check if the line exists, if so, modify it; if not, append it
                found = False
                for i, line in enumerate(lines):
                    if f'vint_set_property(vint_object_find("safe_frame", 0, vint_document_find("{variable_name}")), "anchor"' in line:
                        lines[i] = line_to_append
                        found = True
                        print(f"Modified in {filename}: {line_to_append.strip()}")
                        break

                if not found:
                    if not lines[-1].strip():  # Ensure the last line is empty before appending
                        lines[-1] = line_to_append
                    else:
                        lines.append('\n' + line_to_append)
                    print(f"Added to {filename}: {line_to_append.strip()}")

                file.seek(0)
                file.writelines(lines)
                file.truncate()

# Special handling for hud.lua
def handle_hud_file(directory, aspect_ratio, reference_width):
    file_path = os.path.join(directory, "hud.lua")
    with open(file_path, 'r+') as file:
        lines = file.readlines()

        # Handle special groups like mayhem_grp and vignettes
        lines = handle_special_groups(lines, aspect_ratio, reference_width)

        file.seek(0)
        file.writelines(lines)
        file.truncate()

# Main execution
if __name__ == "__main__":
    # Calculate the aspect ratio
    aspect_ratio = screen_width / screen_height

    # Find the equivalent width for 720 height (relative to 1280x720)
    reference_width = aspect_ratio * 720

    floatv = calculate_offset(reference_width)

    directory = os.getcwd()
    append_or_modify_line_in_lua_files(directory, floatv, lua_files_to_process)

    handle_hud_file(directory, aspect_ratio, reference_width)
